const IMPORT_RSS_NS = "http://purl.org/rss/1.0/";
const IMPORT_OPML_NS = "urn:example.com:opml:1.0:"; // XXX this is made up

function getChildOfName(node,tagname)
{
  var node = node.firstChild;
  while (node){
    if (node.tagName == tagname) return node;
    node = node.nextSibling;
  }
  return null;
}

function getNodeText(node)
{
  var str = "";
  var node = node.firstChild;
  while (node){
    str += node.nodeValue;
    node = node.nextSibling;
  }
  return str;
}

function parseIntoRDF(ds,node,namespace,resource)
{
  var node = node.firstChild;
  while (node){
    if (node.nodeType == node.ELEMENT_NODE){
      RDFAssert(ds,resource,namespace + node.tagName,getNodeText(node));

      var len = node.attributes.length;
      for (var t=0; t<len; t++){
        var attr = node.attributes[t];
        RDFAssert(ds,resource,namespace + attr.name,attr.value);
      }
    }

    node = node.nextSibling;
  }
}

function parseIntoRDFContainer(ds,node,namespace,container)
{
  var node = node.firstChild;
  while (node){
    if (node.nodeType == node.ELEMENT_NODE){
      var resource = RDF.GetAnonymousResource();

      var len = node.attributes.length;
      for (var t=0; t<len; t++){
        var attr = node.attributes[t];
        RDFAssert(ds,resource,namespace + attr.name,attr.value);
      }
      container.AppendElement(resource);
    }

    node = node.nextSibling;
  }
}

function importDatasource(ds,uri,doc)
{
  var subds = null;
  var docElement = doc.documentElement;
  switch (docElement.tagName){
    case "rss":
      subds = createNewDatasource();
      addDatasource(subds);
      importDatasource_rss(subds,uri,docElement);
      break;
    case "opml":
      subds = createNewDatasource();
      addDatasource(subds);
      importDatasource_opml(subds,uri,docElement);
      break;
  }
  return subds;
}

// XXX check namespaces
function importDatasource_rss(ds,uri,docElement)
{
  var channel = RDF.GetResource(uri);
  RDFAssert(ds,channel,RDF_NS + "type",RDF.GetResource(IMPORT_RSS_NS + "channel"));

  var RDFContainerUtils = "@mozilla.org/rdf/container-utils;1";
  RDFContainerUtils = Components.classes[RDFContainerUtils].createInstance(
                         Components.interfaces.nsIRDFContainerUtils);
  var containerRes = RDF.GetAnonymousResource();
  var container = RDFContainerUtils.MakeSeq(ds,containerRes);
  RDFAssert(ds,channel,IMPORT_RSS_NS + "items",containerRes);

  var title = null;

  var node = getChildOfName(docElement,"channel");
  if (node) node = node.firstChild;

  while (node){
    if (node.nodeType == node.ELEMENT_NODE){
      switch (node.tagName){
        case "title":
          title = getNodeText(node)
        case "link":
        case "description":
          RDFAssert(ds,channel,IMPORT_RSS_NS + node.tagName,getNodeText(node));
          break;
        case "pubDate":
          if (node.firstChild){
            // XXX convert date
            RDFAssert(ds,channel,DC_NS + "date",getNodeText(node));
          }
          break;
        case "item":
          var item = importDatasource_rss_item(ds,channel,node,title);
          container.AppendElement(item);
          break;
      }
    }
    node = node.nextSibling;
  }
}

function importDatasource_rss_item(ds,channel,itemnode,title)
{
  var item;
  var node = getChildOfName(channel,"guid");
  if (node && node.firstChild) item = RDF.GetResource(getNodeText(node));
  else item = RDF.GetAnonymousResource();

  RDFAssert(ds,item,RDF_NS + "type",RDF.GetResource(IMPORT_RSS_NS + "item"));

  if (title) RDFAssert(ds,item,DC_NS + "creator",title);

  node = itemnode.firstChild;
  while (node){
    if (node.nodeType == node.ELEMENT_NODE){
      switch (node.tagName){
        case "title":
        case "link":
        case "description":
          if (node.firstChild)
            RDFAssert(ds,item,IMPORT_RSS_NS + node.tagName,getNodeText(node));
          break;
        case "pubDate":
          if (node.firstChild){
            // XXX convert date
            RDFAssert(ds,item,DC_NS + "date",getNodeText(node));
          }
          break;
      }
    }
    node = node.nextSibling;
  }
  return item;
}


function importDatasource_opml(ds,uri,docElement)
{
  var headRes = RDF.GetResource(uri);
  RDFAssert(ds,headRes,RDF_NS + "type",RDF.GetResource(IMPORT_OPML_NS + "opml"));

  var RDFContainerUtils = "@mozilla.org/rdf/container-utils;1";
  RDFContainerUtils = Components.classes[RDFContainerUtils].createInstance(
                         Components.interfaces.nsIRDFContainerUtils);
  var bodyRes = RDF.GetAnonymousResource();
  var container = RDFContainerUtils.MakeSeq(ds,bodyRes);
  RDFAssert(ds,headRes,IMPORT_OPML_NS + "body",bodyRes);

  var head = getChildOfName(docElement,"head")
  parseIntoRDF(ds,head,IMPORT_OPML_NS,headRes);

  var body = getChildOfName(docElement,"body")
  parseIntoRDFContainer(ds,body,IMPORT_OPML_NS,container);
}
